#include "keys.h"
#include "loadconf.h"
#include "events.h"
#include "evilwm.h"
#include "misc.h"

#include <iostream>
#include <stdexcept>

#include <X11/Xutil.h>
#include <X11/keysym.h>

#include <boost/lexical_cast.hpp>

Keys::Keys() {
    ConfMapGenerater cmg;
    cmg.setReportDevice(std::cerr);

	keymap_.push_back(Keymap());
    cmg.generate(getConfigFile("keys"), keymap_[0]);

	mode_ = 0;

	int i = 1;
	while (1) {
		std::string k = "keys" + boost::lexical_cast<std::string>(i);
		std::string kc = getConfigFile(k);
		if (kc == "") break;
		keymap_.push_back(Keymap());
		cmg.generate(kc, keymap_[i]);
		i++;
	}
}

Keys::KeyState::KeyState()
{}

Keys::KeyState::KeyState(const std::string& first)
    : shift_(false), ctrl_(false), alt_(false)
{
    std::string str(first);

    while (str.size() > 1) {
		if (str.substr(0, 2) == "S-") {
			str.erase(0, 2);
			shift_ = true;
		}
		else if (str.substr(0, 2) == "C-") {
			str.erase(0, 2);
			ctrl_ = true;
		}
		else if (str.substr(0, 2) == "M-") {
			str.erase(0, 2);
			alt_ = true;
		}
		else {
			break;
		}
    }

    if (str.size() != 1) {
		if (str == "TAB") key_ = XK_Tab;
		else if (str == "SPC") key_ = XK_space;
		else if (str == "DEL") key_ = XK_Delete;
		else if (str == "RET") key_ = XK_Return;
		else if (str == "ESC") key_ = XK_Escape;
		else if (str == "UP") key_ = XK_Up;
		else if (str == "DOWN") key_ = XK_Down;
		else if (str == "LEFT") key_ = XK_Left;
		else if (str == "RIGHT") key_ = XK_Right;
		else if (str == "INS") key_ = XK_Insert;
		else if (str == "BUTTON1") key_ = XK_Pointer_Button1;
		else if (str == "BUTTON2") key_ = XK_Pointer_Button2;
		else if (str == "BUTTON3") key_ = XK_Pointer_Button3;
		else if (str == "BUTTON4") key_ = XK_Pointer_Button4;
		else if (str == "BUTTON5") key_ = XK_Pointer_Button5;
		else throw std::runtime_error(first + ": unknown key");
    }
    else {
		key_ = XStringToKeysym(str.c_str());
    }
}

bool Keys::KeyState::operator < (const KeyState& rhs) const {
    if (shift_ < rhs.shift_) return true;
    else if (shift_ > rhs.shift_) return false;
    if (ctrl_ < rhs.ctrl_) return true;
    else if (ctrl_ > rhs.ctrl_) return false;
    if (alt_ < rhs.alt_) return true;
    else if (alt_ > rhs.alt_) return false;
    if (key_ < rhs.key_) return true;
    else return false;
}

Keys::Func::Func(const std::string& first)
    : arg_("")
{
    std::string str(first);

    unsigned int s;
    if ((s = str.find(" ")) != std::string::npos) {
		arg_ = str.substr(s+1);
		str.erase(s);
    }
    if (str == "MOVE_WINDOW_X") func_ = &move_window_x;
    else if (str == "MOVE_WINDOW_Y") func_ = &move_window_y;
    else if (str == "MOVE_WINDOW_UPRIGHT") func_ = &move_window_upright;
    else if (str == "MOVE_WINDOW_UPLEFT") func_ = &move_window_upleft;
    else if (str == "MOVE_WINDOW_DOWNRIGHT") func_ = &move_window_downright;
    else if (str == "MOVE_WINDOW_DOWNLEFT") func_ = &move_window_downleft;
    else if (str == "DELETE_WINDOW") func_ = &delete_window;
    else if (str == "INFO_WINDOW") func_ = &info_window;
    else if (str == "MAXIMISE_WINDOW") func_ = &maximise_window;
    else if (str == "MAXIMISE_WINDOW_VERT") func_ = &maximise_window_vert;
    else if (str == "FIX_WINDOW") func_ = &fix_window;
    else if (str == "EXEC_COMMAND") func_ = &exec_command;
    else if (str == "NEXT_FOCUS") func_ = &next_focus;
    else if (str == "SWITCH_VDESK") func_ = &switch_vdesk;
    else if (str == "NEXT_VDESK") func_ = &next_vdesk;
    else if (str == "PREV_VDESK") func_ = &prev_vdesk;
    else if (str == "FIX_NEXT_VDESK") func_ = &fix_next_vdesk;
    else if (str == "FIX_PREV_VDESK") func_ = &fix_prev_vdesk;
    else if (str == "UP_FOCUS") func_ = &up_focus;
    else if (str == "DOWN_FOCUS") func_ = &down_focus;
    else if (str == "LEFT_FOCUS") func_ = &left_focus;
    else if (str == "RIGHT_FOCUS") func_ = &right_focus;
    else if (str == "UPLEFT_FOCUS") func_ = &upleft_focus;
    else if (str == "UPRIGHT_FOCUS") func_ = &upright_focus;
    else if (str == "DOWNLEFT_FOCUS") func_ = &downleft_focus;
    else if (str == "DOWNRIGHT_FOCUS") func_ = &downright_focus;
    else if (str == "WM_QUIT") func_ = &wm_quit;
    else if (str == "WM_RESTART") func_ = &wm_restart;
	else if (str == "KEY_MARK") func_ = &mark_client_str;
	else if (str == "GOTO_MARK") func_ = &goto_mark_str;
	else if (str == "WARP_POINTER_X") func_ = &warp_pointer_x;
	else if (str == "WARP_POINTER_Y") func_ = &warp_pointer_y;
	else if (str == "SET_MODE") func_ = &set_mode;
	else if (str == "LOWER_WINDOW") func_ = &lower_window;
	else if (str == "RAISE_WINDOW") func_ = &raise_window;
	else if (str == "MOUSE_MOVE") func_ = &mouse_move;
	else if (str == "MOUSE_RESIZE") func_ = &mouse_resize;
	else if (str == "RESIZE_WINDOW_X") func_ = &resize_window_x;
	else if (str == "RESIZE_WINDOW_Y") func_ = &resize_window_y;
    else std::runtime_error(first + ": unknown command");
}

void Keys::run(XKeyPressedEvent* ke) const {
    Keymap::const_iterator ite = keymap_[mode_].find(KeyState(ke));
    if (ite != keymap_[mode_].end()) {
		ite->second.run();
    }
}

void Keys::run(XButtonEvent* ke) const {
    Keymap::const_iterator ite = keymap_[mode_].find(KeyState(ke));
    if (ite != keymap_[mode_].end()) {
		ite->second.run();
    }
}

Keys::KeyState::KeyState(XKeyPressedEvent* ke)
    : shift_(false), ctrl_(false), alt_(false)
{
    key_= XKeycodeToKeysym(dpy, ke->keycode, 0);
    if (ke->state & ShiftMask) shift_ = true;
}

Keys::KeyState::KeyState(XButtonEvent* be)
{
	shift_ = be->state & ShiftMask;
	ctrl_ = be->state & ControlMask;
	alt_ = be->state & Mod1Mask;
	if (be->button == Button1) key_ = XK_Pointer_Button1;
	else if (be->button == Button2) key_ = XK_Pointer_Button2;
	else if (be->button == Button3) key_ = XK_Pointer_Button3;
	else if (be->button == Button4) key_ = XK_Pointer_Button4;
	else if (be->button == Button5) key_ = XK_Pointer_Button5;
}

void Keys::Func::run() const {
    (*func_)(arg_);
}

