#ifndef __KEYBOARDWM_KEYS_H__
#define __KEYBOARDWM_KEYS_H__

#include <string>
#include <map>
#include <vector>

#include <X11/Xlib.h>

class Keys {
public:
    class Func {
    private:
		typedef void (*Func_)(const std::string&);

    public:
		Func() {}
		Func(const std::string& str);
		Func(Func_ func) : func_(func) {}
		operator Func_() { return func_; }
		void run() const;

    private:
		Func_ func_;
		std::string arg_;
    };

public:
    Keys();

    void run(XKeyEvent* ke) const;
    void run(XButtonEvent* ke) const;

	void setMode(int m) { mode_ = m; }

private:
    class KeyState {
    public:
		KeyState();
		KeyState(const std::string& first);
		explicit KeyState(XKeyEvent* ke);
		explicit KeyState(XButtonEvent* be);

		bool operator < (const KeyState& rhs) const;

    private:
		KeySym key_;
		bool shift_;
		bool ctrl_;
		bool alt_;
    };

private:
    typedef std::map<KeyState, Func> Keymap;
    std::vector<Keymap> keymap_;

	size_t mode_;

};

#endif // ! __KEYBOARDWM_KEYS_H__

