/* evilwm - Minimalist Window Manager for X
 * Copyright (C) 1999-2000 Ciaran Anscomb <ciarana@rd.bbc.co.uk>
 * see README for license and other details. */

#include "evilwm.h"
#include "misc.h"

#include <X11/Xproto.h>
#include <stdlib.h>
#include <stdarg.h>
#include <signal.h>
#include <sys/wait.h>
#include <unistd.h>
#include <stdio.h>
#include <iostream>
#include <fstream>
#include <string.h>
#include <errno.h>
#include <sstream>
#include <vector>

void throwAllUnmapEvent() {
	XEvent ev;
    // throw all event
	while (XCheckTypedEvent(dpy, UnmapNotify, &ev)) {}
}

std::string getConfigFile(const std::string& name) {
    std::string file;
    std::ifstream is;

    char* home = getenv("HOME");
    if (home) {
		file = std::string(home) + "/.sevilwm/" + name;
		is.open(file.c_str());
		if (is && !is.eof()) return file;
		is.close();
    }

    file = CONF_DIR + name;
    is.open(file.c_str());
    if (is && !is.eof()) return file;

	file = name;
	is.open(file.c_str());
	if (is && !is.eof()) return file;

    return "";
}

void spawn(const std::string& str) {
    std::istringstream iss(str);
    std::vector<std::string> strs;
    int i = 0;
    while (iss) {
		std::string tmp;
		iss >> tmp;
		strs.push_back(tmp);
		i++;
    }
    const char* cmd = strs[0].c_str();

    char* argv[100];
    std::vector<std::string>::iterator ite;
    i = 0;
    for (ite = strs.begin(); ite != strs.end(); ite++) {
		if (ite->empty()) continue;
		argv[i] = const_cast<char*>(ite->c_str());
		i++;
    }
    argv[i] = 0;

    pid_t pid = fork();

    if (pid == 0) {
		execvp(cmd, argv);
		std::cerr << "error: " << strerror(errno) << std::endl;
    }
}

void handle_signal(int signo) {
    if (signo == SIGCHLD) {
		wait(NULL);
		return;
    }

    quit_nicely();
}

void cleanup() {
    while(head_client) remove_client(head_client, QUITTING);
    XSetInputFocus(dpy, PointerRoot, RevertToPointerRoot, CurrentTime);
    XInstallColormap(dpy, DefaultColormap(dpy, screen));
    XCloseDisplay(dpy);
}

void restart() {
    cleanup();
    execl("/bin/sh", "sh", "-c", window_manager_name, 0);
}

void quit_nicely() {
    cleanup();
    exit(0);
}

int handle_xerror(Display *, XErrorEvent *e) {
    Client *c = find_client(e->resourceid);

    /* if (e->error_code == BadAccess && e->resourceid == root) { */
    if (e->error_code == BadAccess &&
		e->request_code == X_ChangeWindowAttributes) {
#ifdef STDIO
		fprintf(stderr, "root window unavailable (maybe another wm is running?)\n");
#endif
		exit(1);
    }
    fprintf(stderr, "XError %x %d ", e->error_code, e->request_code);
    if (c) {
#ifdef XDEBUG
		fprintf(stderr, "(removing client)\n");
#endif
		remove_client(c, NOT_QUITTING);
    }
    return 0;
}

int ignore_xerror(Display *, XErrorEvent *) {
    return 0;
}

#ifdef DEBUG
void dump_clients() {
    Client *c;

    for (c = head_client; c; c = c->next)
		fprintf(stderr, "MISC: (%d, %d) @ %d,%d\n", wm_state(c),
				c->ignore_unmap, c->x, c->y);
}
#endif /* DEBUG */

#ifdef SANITY
void sanity_check() {
    Client *c;
    int count = 0;

    for (c = head_client; c; c = c->next)
		count++;
    if (count != tracked_count) {
		fprintf(stderr, "misc:sanity_check() failed - should be %d clients, I count %d\n", tracked_count, count);
		exit(1);
    }
}
#endif /* SANITY */
